/**
 * DevExtreme (esm/renovation/ui/scroll_view/scrollable_utils.js)
 * Version: 21.1.0 (build 21089-0329)
 * Build date: Tue Mar 30 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import _extends from "@babel/runtime/helpers/esm/extends";
import {
    isNumeric,
    isDefined,
    isPlainObject
} from "../../../core/utils/type";
import getScrollRtlBehavior from "../../../core/utils/scroll_rtl_behavior";
import {
    camelize
} from "../../../core/utils/inflector";
import getElementComputedStyle from "../../utils/get_computed_style";
import {
    toNumber
} from "../../utils/type_conversion";
import {
    ensureDefined
} from "../../../core/utils/common";
export var SCROLL_LINE_HEIGHT = 40;
export var DIRECTION_VERTICAL = "vertical";
export var DIRECTION_HORIZONTAL = "horizontal";
export var DIRECTION_BOTH = "both";
export var SCROLLABLE_SIMULATED_CLASS = "dx-scrollable-simulated";
export var SCROLLABLE_CONTENT_CLASS = "dx-scrollable-content";
export var SCROLLABLE_WRAPPER_CLASS = "dx-scrollable-wrapper";
export var SCROLLABLE_CONTAINER_CLASS = "dx-scrollable-container";
export var SCROLLVIEW_CONTENT_CLASS = "dx-scrollview-content";
export var SCROLLVIEW_BOTTOM_POCKET_CLASS = "dx-scrollview-bottom-pocket";
export var SCROLLABLE_DISABLED_CLASS = "dx-scrollable-disabled";
export var SCROLLABLE_SCROLLBAR_SIMULATED = "dx-scrollable-scrollbar-simulated";
export var SCROLLABLE_SCROLLBARS_HIDDEN = "dx-scrollable-scrollbars-hidden";
export var SCROLLABLE_SCROLLBARS_ALWAYSVISIBLE = "dx-scrollable-scrollbars-alwaysvisible";
export var SCROLLABLE_SCROLLBAR_CLASS = "dx-scrollable-scrollbar";
export function getElementWidth(element) {
    var _getElementComputedSt;
    return toNumber(null === (_getElementComputedSt = getElementComputedStyle(element)) || void 0 === _getElementComputedSt ? void 0 : _getElementComputedSt.width)
}
export function getElementHeight(element) {
    var _getElementComputedSt2;
    return toNumber(null === (_getElementComputedSt2 = getElementComputedStyle(element)) || void 0 === _getElementComputedSt2 ? void 0 : _getElementComputedSt2.height)
}
export function getElementStyle(name, element) {
    var computedStyle = getElementComputedStyle(element) || {};
    return computedStyle[name]
}
export function ensureLocation(location) {
    if (isNumeric(location)) {
        return {
            left: location,
            top: location
        }
    }
    return _extends({
        top: 0,
        left: 0
    }, location)
}

function getRelativeLocation(element) {
    var result = {
        top: 0,
        left: 0
    };
    var targetElement = element;
    while (!targetElement.matches(".".concat(SCROLLABLE_CONTENT_CLASS))) {
        result.top += targetElement.offsetTop;
        result.left += targetElement.offsetLeft;
        targetElement = targetElement.offsetParent
    }
    return result
}
export class ScrollDirection {
    constructor(direction) {
        this.DIRECTION_HORIZONTAL = "horizontal";
        this.DIRECTION_VERTICAL = "vertical";
        this.DIRECTION_BOTH = "both";
        this.direction = null !== direction && void 0 !== direction ? direction : DIRECTION_VERTICAL
    }
    get isHorizontal() {
        return this.direction === DIRECTION_HORIZONTAL || this.direction === DIRECTION_BOTH
    }
    get isVertical() {
        return this.direction === DIRECTION_VERTICAL || this.direction === DIRECTION_BOTH
    }
    get isBoth() {
        return this.direction === DIRECTION_BOTH
    }
}

function getMaxScrollOffset(dimension, containerRef) {
    return containerRef["scroll".concat(camelize(dimension, true))] - containerRef["client".concat(camelize(dimension, true))]
}
export function getBoundaryProps(direction, scrollOffset, element) {
    var {
        left: left,
        top: top
    } = scrollOffset;
    var boundaryProps = {};
    var {
        isHorizontal: isHorizontal,
        isVertical: isVertical
    } = new ScrollDirection(direction);
    if (isHorizontal) {
        boundaryProps.reachedLeft = left <= 0;
        boundaryProps.reachedRight = Math.round(left) >= getMaxScrollOffset("width", element)
    }
    if (isVertical) {
        boundaryProps.reachedTop = top <= 0;
        boundaryProps.reachedBottom = top >= getMaxScrollOffset("height", element)
    }
    return boundaryProps
}
export function getContainerOffsetInternal(element) {
    return {
        left: element.scrollLeft,
        top: element.scrollTop
    }
}

function getScrollBarSize(dimension, containerRef) {
    return containerRef["offset".concat(dimension)] - containerRef["client".concat(dimension)]
}

function needNormalizeCoordinate(prop, rtlEnabled) {
    return true === rtlEnabled && "left" === prop
}
export function normalizeCoordinate(prop, coordinate, rtlEnabled) {
    return needNormalizeCoordinate(prop, rtlEnabled) && getScrollRtlBehavior().positive ? -1 * coordinate : coordinate
}
export function getPublicCoordinate(prop, coordinate, containerRef, rtlEnabled) {
    return needNormalizeCoordinate(prop, rtlEnabled) ? getMaxScrollOffset("width", containerRef) + normalizeCoordinate(prop, coordinate, rtlEnabled) : coordinate
}
export function normalizeLocation(location, direction) {
    if (isPlainObject(location)) {
        var left = ensureDefined(location.left, location.x);
        var top = ensureDefined(location.top, location.y);
        return {
            left: isDefined(left) ? -left : void 0,
            top: isDefined(top) ? -top : void 0
        }
    }
    var {
        isHorizontal: isHorizontal,
        isVertical: isVertical
    } = new ScrollDirection(direction || "vertical");
    return {
        left: isHorizontal ? -location : void 0,
        top: isVertical ? -location : void 0
    }
}

function getElementLocationInternal(element, prop, offset, direction, containerRef, rtlEnabled) {
    var dimension = direction === DIRECTION_VERTICAL ? "Height" : "Width";
    var relativeLocation = getRelativeLocation(element)[prop];
    var scrollBarSize = getScrollBarSize(dimension, containerRef);
    var containerSize = containerRef["offset".concat(dimension)];
    var elementOffset = element["offset".concat(dimension)];
    var offsetStart = offset[prop];
    var offsetEnd = offset[direction === DIRECTION_VERTICAL ? "bottom" : "right"] || 0;
    var containerLocation = normalizeCoordinate(prop, getContainerOffsetInternal(containerRef)[prop], rtlEnabled);
    if (relativeLocation < containerLocation + offsetStart) {
        if (elementOffset < containerSize - offsetStart - offsetEnd) {
            return relativeLocation - offsetStart
        }
        return relativeLocation + elementOffset - containerSize + offsetEnd + scrollBarSize
    }
    if (relativeLocation + elementOffset >= containerLocation + containerSize - offsetEnd - scrollBarSize) {
        if (elementOffset < containerSize - offsetStart - offsetEnd) {
            return relativeLocation + elementOffset + scrollBarSize - containerSize + offsetEnd
        }
        return relativeLocation - offsetStart
    }
    return containerLocation
}
export function getElementLocation(element, offset, direction, containerRef, rtlEnabled) {
    var prop = direction === DIRECTION_VERTICAL ? "top" : "left";
    var location = normalizeCoordinate(prop, getElementLocationInternal(element, prop, offset, direction, containerRef, rtlEnabled), rtlEnabled);
    return getPublicCoordinate(prop, location, containerRef, rtlEnabled)
}
export function updateAllowedDirection(allowedDirections, direction) {
    var {
        isBoth: isBoth,
        isHorizontal: isHorizontal,
        isVertical: isVertical
    } = new ScrollDirection(direction);
    if (isBoth && allowedDirections.vertical && allowedDirections.horizontal) {
        return DIRECTION_BOTH
    }
    if (isHorizontal && allowedDirections.horizontal) {
        return DIRECTION_HORIZONTAL
    }
    if (isVertical && allowedDirections.vertical) {
        return DIRECTION_VERTICAL
    }
    return
}
